// This source code is a part of Project Violet.
// Copyright (C) 2020-2022. violet-team. Licensed under the Apache-2.0 License.

import 'dart:collection';

import 'package:violet/component/image_provider.dart';

class ProviderManager {
  static HashMap<int, VioletImageProvider> _ids =
      HashMap<int, VioletImageProvider>();
  static HashMap<int, bool> _dirty = HashMap<int, bool>();

  static bool isExists(int id) {
    return _ids.containsKey(id);
  }

  static void insert(int id, VioletImageProvider url) {
    _dirty[id] = false;
    _ids[id] = url;
  }

  static VioletImageProvider getIgnoreDirty(int id) {
    return _ids[id];
  }

  static Future<VioletImageProvider> get(int id) async {
    if (_dirty[id]) {
      _dirty[id] = false;
      await _ids[id].refresh();
    }
    return _ids[id];
  }

  static void clear() {
    _ids.clear();
  }

  static void refresh() {
    for (var v in _dirty.entries) {
      if (_ids[v.key].isRefreshable()) _dirty[v.key] = true;
    }
  }
}
